@@ -5,6 +5,10 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
+"""
+The Loki source-to-source translation package for Fortran codes.
+"""
+
 from importlib.metadata import version, PackageNotFoundError
 
 # Import the global configuration map
@@ -22,12 +26,12 @@ from loki.scope import *  # noqa
 from loki.tools import *  # noqa
 from loki.logging import *  # noqa
 from loki.backend import *  # noqa
-from loki.transform import * # noqa
 from loki.build import *  # noqa  # pylint: disable=redefined-builtin
 from loki.batch import *  # noqa
 from loki.lint import *  # noqa
 from loki.analyse import *  # noqa
 from loki.dimension import *  # noqa
+from loki.transformations import *  # noqa
 
 
 try:
