@@ -33,7 +33,7 @@ def is_loki_pragma(pragma, starts_with=None):
 
     Parameters
     ----------
-    pragma : :any:`Pragma` or `list`/`tuple` of `ir.Pragma` or `None`
+    pragma : :any:`Pragma` or `list`/`tuple` of :any:`Pragma` or `None`
         the pragma or list of pragmas to check.
     starts_with : str, optional
         the keyword the pragma content must start with.
@@ -67,10 +67,14 @@ class PragmaParameters:
 
         .. note::
             This allows nested parenthesis by matching pairs of
-            parantheses starting at the end by pushing and popping
+            parentheses starting at the end by pushing and popping
             from a stack.
         """
         string = cls._pattern_quoted_string.sub('', string)
+        if not string.strip():
+            # Early bail-out on empty strings
+            return {}
+
         p_open = [match.start() for match in cls._pattern_opening_parenthesis.finditer(string)]
         p_close = [match.start() for match in cls._pattern_closing_parenthesis.finditer(string)]
         assert len(p_open) == len(p_close)
@@ -101,18 +105,21 @@ class PragmaParameters:
             spans.append(p_spans.pop())
         if p_spans:
             spans += p_spans[::-1]
+
+        # Build the list of parameters from the matched spans
         parameters = defaultdict(list)
-        if not spans and string.strip():
-            for key in string.strip().split(' '):
-                if key != '':
-                    parameters[key].append(None)
         for i, span in enumerate(spans):
             keys = string[spans[i-1][1]+1 if i>=1 else 0:span[0]].strip().split(' ')
-            if len(keys) > 1:
-                for key in keys[:-1]:
-                    if key != '':
-                        parameters[key].append(None)
+            for key in keys[:-1]:
+                if key:
+                    parameters[key].append(None)
             parameters[keys[-1]].append(string[span[0]+1:span[1]])
+
+        # Tail handling (including strings without any matched spans)
+        tail_span = spans[-1][1] + 1 if spans else 0
+        for key in string[tail_span:].strip().split(' '):
+            if key != '':
+                parameters[key].append(None)
         parameters = {k: v if len(v) > 1 else v[0] for k, v in parameters.items()}
         return parameters
 
@@ -149,6 +156,8 @@ def get_pragma_parameters(pragma, starts_with=None, only_loki_pragmas=True):
         if only_loki_pragmas and p.keyword.lower() != 'loki':
             continue
         content = p.content or ''
+        # Remove any line-continuation markers
+        content = content.replace('&', '')
         if starts_with is not None:
             if not content.lower().startswith(starts_with.lower()):
                 continue
@@ -347,7 +356,7 @@ def attach_pragmas(ir, node_type, attach_pragma_post=True):
         the root of (a section of the) intermediate representation in which
         pragmas are to be attached.
     node_type : list
-        the (list of) :any:`ir.Node` types pragmas should be attached to.
+        the (list of) :any:`Node` types pragmas should be attached to.
     attach_pragma_post : bool, optional
         process ``pragma_post`` attachments.
     """
@@ -375,7 +384,7 @@ def detach_pragmas(ir, node_type, detach_pragma_post=True):
         the root node of the (section of the) intermediate representation
         in which pragmas are to be detached.
     node_type :
-        the (list of) :any:`ir.Node` types that pragmas should be detached from.
+        the (list of) :any:`Node` types that pragmas should be detached from.
     detach_pragma_post: bool, optional
         process ``pragma_post`` attachments.
     """
@@ -433,7 +442,7 @@ def pragmas_attached(module_or_routine, node_type, attach_pragma_post=True):
     module_or_routine : :any:`Module` or :any:`Subroutine`
         the program unit in which pragmas are to be inlined.
     node_type :
-        the (list of) :any:`ir.Node` types, that pragmas should be
+        the (list of) :any:`Node` types, that pragmas should be
         attached to.
     attach_pragma_post : bool, optional
         process ``pragma_post`` attachments.
