@@ -5,23 +5,18 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
-from pathlib import Path
 import pytest
 import numpy as np
 
 from loki import Subroutine
+from loki.backend import fgen
 from loki.build import jit_compile, clean_test
-from loki.frontend import available_frontends, OMNI
+from loki.frontend import available_frontends, OMNI, OFP
 from loki.ir import nodes as ir, FindNodes
 
 
-@pytest.fixture(scope='module', name='here')
-def fixture_here():
-    return Path(__file__).parent
-
-
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_loop_nest_fixed(here, frontend):
+def test_loop_nest_fixed(tmp_path, frontend):
     """
     Test basic loops and reductions with fixed sizes.
 
@@ -54,7 +49,7 @@ subroutine loop_nest_fixed(in1, in2, out1, out2)
   end do
 end subroutine loop_nest_fixed
 """
-    filepath = here/(f'control_flow_loop_nest_fixed_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_loop_nest_fixed_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='loop_nest_fixed')
 
@@ -70,7 +65,7 @@ end subroutine loop_nest_fixed
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_loop_nest_variable(here, frontend):
+def test_loop_nest_variable(tmp_path, frontend):
     """
     Test basic loops and reductions with passed sizes.
 
@@ -104,7 +99,7 @@ subroutine loop_nest_variable(dim1, dim2, in1, in2, out1, out2)
   end do
 end subroutine loop_nest_variable
 """
-    filepath = here/(f'control_flow_loop_nest_variable_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_loop_nest_variable_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='loop_nest_variable')
 
@@ -120,7 +115,7 @@ end subroutine loop_nest_variable
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_loop_scalar_logical_expr(here, frontend):
+def test_loop_scalar_logical_expr(tmp_path, frontend):
     """
     Test a while loop with a logical expression as condition.
     """
@@ -135,7 +130,7 @@ subroutine loop_scalar_logical_expr(outvar)
   end do
 end subroutine loop_scalar_logical_expr
 """
-    filepath = here/(f'control_flow_loop_scalar_logical_expr_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_loop_scalar_logical_expr_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='loop_scalar_logical_expr')
 
@@ -145,7 +140,7 @@ end subroutine loop_scalar_logical_expr
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_loop_unbounded(here, frontend):
+def test_loop_unbounded(tmp_path, frontend):
     """
     Test unbounded loops.
     """
@@ -163,7 +158,7 @@ subroutine loop_unbounded(out)
   enddo
 end subroutine loop_unbounded
 """
-    filepath = here/(f'control_flow_loop_unbounded_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_loop_unbounded_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='loop_unbounded')
 
@@ -173,7 +168,7 @@ end subroutine loop_unbounded
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_loop_labeled_continue(here, frontend):
+def test_loop_labeled_continue(tmp_path, frontend):
     """
     Test labeled loops with continue statement.
 
@@ -193,7 +188,7 @@ subroutine loop_labeled_continue(out)
 101 continue
 end subroutine loop_labeled_continue
 """
-    filepath = here/(f'control_flow_loop_labeled_continue_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_loop_labeled_continue_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
 
     if frontend != OMNI:  # OMNI doesn't read the Loop label...
@@ -207,7 +202,7 @@ end subroutine loop_labeled_continue
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_inline_conditionals(here, frontend):
+def test_inline_conditionals(tmp_path, frontend):
     """
     Test the use of inline conditionals.
     """
@@ -224,7 +219,7 @@ subroutine inline_conditionals(in1, in2, out1, out2)
   if (in2 > 5) out2 = 5
 end subroutine inline_conditionals
 """
-    filepath = here/(f'control_flow_inline_conditionals_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_inline_conditionals_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='inline_conditionals')
 
@@ -239,7 +234,7 @@ end subroutine inline_conditionals
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_multi_body_conditionals(here, frontend):
+def test_multi_body_conditionals(tmp_path, frontend):
     fcode = """
 subroutine multi_body_conditionals(in1, out1, out2)
   integer, intent(in) :: in1
@@ -263,7 +258,7 @@ subroutine multi_body_conditionals(in1, out1, out2)
   end if
 end subroutine multi_body_conditionals
 """
-    filepath = here/(f'control_flow_multi_body_conditionals_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_multi_body_conditionals_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
 
     conditionals = FindNodes(ir.Conditional).visit(routine.body)
@@ -288,7 +283,7 @@ end subroutine multi_body_conditionals
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_goto_stmt(here, frontend):
+def test_goto_stmt(tmp_path, frontend):
     fcode = """
 subroutine goto_stmt(var)
   implicit none
@@ -300,7 +295,7 @@ subroutine goto_stmt(var)
   var = 7
 end subroutine goto_stmt
 """
-    filepath = here/(f'control_flow_goto_stmt_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_goto_stmt_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='goto_stmt')
 
@@ -310,7 +305,7 @@ end subroutine goto_stmt
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_select_case(here, frontend):
+def test_select_case(tmp_path, frontend):
     fcode = """
 subroutine select_case(cmd, out1)
   implicit none
@@ -329,7 +324,7 @@ subroutine select_case(cmd, out1)
   end select
 end subroutine select_case
 """
-    filepath = here/(f'control_flow_select_case_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_select_case_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='select_case')
 
@@ -341,7 +336,7 @@ end subroutine select_case
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_select_case_nested(here, frontend):
+def test_select_case_nested(tmp_path, frontend):
     fcode = """
 subroutine select_case(cmd, out1)
   implicit none
@@ -376,7 +371,7 @@ subroutine select_case(cmd, out1)
   end select
 end subroutine select_case
 """
-    filepath = here/(f'control_flow_select_case_nested_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_select_case_nested_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='select_case')
 
@@ -390,7 +385,7 @@ end subroutine select_case
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_cycle_stmt(here, frontend):
+def test_cycle_stmt(tmp_path, frontend):
     fcode = """
 subroutine cycle_stmt(var)
   implicit none
@@ -404,7 +399,7 @@ subroutine cycle_stmt(var)
   end do
 end subroutine cycle_stmt
 """
-    filepath = here/(f'control_flow_cycle_stmt_{frontend}.f90')
+    filepath = tmp_path/(f'control_flow_cycle_stmt_{frontend}.f90')
     routine = Subroutine.from_source(fcode, frontend=frontend)
     function = jit_compile(routine, filepath=filepath, objname='cycle_stmt')
 
@@ -499,3 +494,208 @@ END FUNCTION FUNC
     assert conditionals[1].body[-1].text.upper() == 'RETURN'
     assert isinstance(conditionals[1].else_body[-1], ir.Intrinsic)
     assert conditionals[1].else_body[-1].text.upper() == 'RETURN'
+
+
+@pytest.mark.parametrize('frontend', available_frontends(
+        xfail=[(OMNI, 'Renames index variable to omnitmp000')]
+))
+def test_single_line_forall_stmt(tmp_path, frontend):
+    fcode = """
+subroutine forall_stmt(n, a)
+    implicit none
+    integer, parameter :: jprb = selected_real_kind(13,300)
+    integer, intent(in) :: n
+    real(kind=jprb), dimension(n, n), intent(inout) :: a
+    integer :: i
+
+    ! Create a diagonal square matrix
+    forall (i=1:n)  a(i, i) = 1
+end subroutine forall_stmt
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+
+    # Check generated IR for the Forall statement
+    statements = FindNodes(ir.Forall).visit(routine.ir)
+    assert len(statements) == 1
+    # Check the i=1:n bound
+    assert len(statements[0].named_bounds) == 1
+    bound_var, bound_range = statements[0].named_bounds[0]
+    assert bound_var.name == 'i'
+    assert bound_range == '1:n'
+    # Check the a(i, i) = 1 assignment
+    assignments = FindNodes(ir.Assignment).visit(statements[0])
+    assert len(assignments) == 1, "Single-line FORALL statement must have only one assignment"
+    assert assignments[0].lhs == "a(i, i)"  # Assign to array `a`
+    assert assignments[0].rhs == '1'  # Assign 1 on the diagonal
+
+    # Check execution and produced results
+    filepath = tmp_path/f'single_line_forall_stmt_{frontend}.f90'
+    fun_forall_stmt = jit_compile(routine, filepath=filepath, objname="forall_stmt")
+    n = 3
+    a = np.zeros((n, n), order="F")
+    fun_forall_stmt(n, a)
+    assert (a == [[1.0, 0.0, 0.0],
+                  [0.0, 1.0, 0.0],
+                  [0.0, 0.0, 1.0]]).all()
+    n = 5
+    a = np.empty((n, n), order="F")
+    a.fill(3.0)
+    fun_forall_stmt(n, a)
+    assert (a == [[1.0, 3.0, 3.0, 3.0, 3.0],
+                  [3.0, 1.0, 3.0, 3.0, 3.0],
+                  [3.0, 3.0, 1.0, 3.0, 3.0],
+                  [3.0, 3.0, 3.0, 1.0, 3.0],
+                  [3.0, 3.0, 3.0, 3.0, 1.0]]).all()
+
+    # Check the fgen code generation
+    expected_fcode = "FORALL(i = 1:n) a(i, i) = 1"
+    assert fgen(statements[0]) == expected_fcode
+    assert expected_fcode in routine.to_fortran()
+
+
+@pytest.mark.parametrize('frontend', available_frontends(
+        xfail=[(OMNI, 'Renames index variable to omnitmp000')]
+))
+def test_single_line_forall_masked_stmt(tmp_path, frontend):
+    fcode = """
+subroutine forall_masked_stmt(n, a, b)
+    implicit none
+    integer, parameter :: jprb = selected_real_kind(13,300)
+    integer, intent(in) :: n
+    real(kind=jprb), dimension(n, n), intent(inout) :: a, b
+    integer :: i, j
+
+    forall(i = 1:n, j = 1:n, a(i, j) .ne. 0.0) b(i, j) = 1.0 / a(i, j)
+end subroutine forall_masked_stmt
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+
+    # Check generated IR for the Forall statement
+    statements = FindNodes(ir.Forall).visit(routine.ir)
+    assert len(statements) == 1
+    assert len(statements[0].named_bounds) == 2
+    # Check the i=1:n bound
+    bound_var, bound_range = statements[0].named_bounds[0]
+    assert bound_var == "i"
+    assert bound_range == '1:n'
+    # Check the j=1:n bound
+    bound_var, bound_range = statements[0].named_bounds[1]
+    assert bound_var == "j"
+    assert bound_range == '1:n'
+    # Check the array mask
+    mask = statements[0].mask
+    assert statements[0].mask == 'a(i, j) != 0.0'
+    # Quickly check assignment
+    assignments = FindNodes(ir.Assignment).visit(statements[0])
+    assert len(assignments) == 1
+    assert assignments[0].lhs.name == "b" and len(assignments[0].lhs.dimensions) == 2
+    assert assignments[0].rhs == '1.0 / a(i, j)'
+
+    # Check execution and produced results
+    filepath = tmp_path / (f'single_line_forall_masked_stmt_{frontend}.f90')
+    fun_forall_masked_stmt = jit_compile(routine, filepath=filepath, objname="forall_masked_stmt")
+    n = 3
+    a = np.array([[2.0, 0.0, 2.0],
+                  [0.0, 4.0, 0.0],
+                  [10.0, 10.0, 0.0]], order="F")
+    b = np.zeros((n, n), order="F")
+    fun_forall_masked_stmt(n, a, b)
+    assert (b == [[0.5, 0.0, 0.5], [0, 0.25, 0], [0.1, 0.1, 0]]).all()
+
+    # Check the fgen code generation
+    expected_fcode = "FORALL(i = 1:n, j = 1:n, a(i, j) /= 0.0) b(i, j) = 1.0 / a(i, j)"
+    assert fgen(statements[0]) == expected_fcode
+    assert expected_fcode in routine.to_fortran()
+
+
+@pytest.mark.parametrize('frontend', available_frontends(xfail=[
+    (OMNI, 'Renames index variable to omnitmp000'),
+    (OFP, 'Parser fails to parse'),
+]))
+def test_multi_line_forall_construct(tmp_path, frontend):
+    fcode = """
+subroutine forall_construct(n, c, d)
+    implicit none
+    integer, parameter :: jprb = selected_real_kind(13,300)
+    integer, intent(in) :: n
+    real(kind=jprb), dimension(n, n), intent(inout) :: c, d
+    integer :: i, j
+
+    forall(i = 3:n - 2, j = 3:n - 2)
+        c(i, j) = c(i, j + 2) + c(i, j - 2) + c(i + 2, j) + c(i - 2, j)
+        d(i, j) = c(i, j)
+    end forall
+end subroutine forall_construct
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+
+    # Check generated IR for the Forall statement
+    statements = FindNodes(ir.Forall).visit(routine.ir)
+    assert len(statements) == 1
+    assert len(statements[0].named_bounds) == 2
+    # Check the i=3:(n-2) bound
+    bound_var, bound_range = statements[0].named_bounds[0]
+    assert bound_var.name == "i"
+    assert bound_range == '3:n-2'
+    # Check the j=3:(n-2) bound
+    bound_var, bound_range = statements[0].named_bounds[1]
+    assert bound_var.name == "j"
+    assert bound_range == '3:n-2'
+    # Check assignments
+    assignments = FindNodes(ir.Assignment).visit(statements[0])
+    assert len(assignments) == 2
+    # Quickly check first assignment
+    assert assignments[0].lhs == 'c(i, j)'
+    assert assignments[0].rhs == 'c(i, j + 2) + c(i, j - 2) + c(i + 2, j) + c(i - 2, j)'
+    # Check the second assignment
+    assert assignments[1].lhs == 'd(i, j)'
+    assert assignments[1].rhs == 'c(i, j)'
+
+    filepath = tmp_path / (f'multi_line_forall_construct_{frontend}.f90')
+    fun_forall_construct = jit_compile(routine, filepath=filepath, objname="forall_construct")
+    n = 6
+    c = np.zeros((n, n), order="F")
+    c.fill(1)
+    d = np.zeros((n, n), order="F")
+    fun_forall_construct(n, c, d)
+    assert (c == [[1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
+                  [1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
+                  [1.0, 1.0, 4.0, 4.0, 1.0, 1.0],
+                  [1.0, 1.0, 4.0, 4.0, 1.0, 1.0],
+                  [1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
+                  [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]]).all()
+    assert (d == [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                  [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                  [0.0, 0.0, 4.0, 4.0, 0.0, 0.0],
+                  [0.0, 0.0, 4.0, 4.0, 0.0, 0.0],
+                  [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                  [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]]).all()
+
+    # Check the fgen code generation
+    regenerated_code = routine.to_fortran().split("\n")
+    assert regenerated_code[7].strip() == "FORALL(i = 3:n - 2, j = 3:n - 2)"
+    assert regenerated_code[8].strip() == "c(i, j) = c(i, j + 2) + c(i, j - 2) + c(i + 2, j) + c(i - 2, j)"
+    assert regenerated_code[9].strip() == "d(i, j) = c(i, j)"
+    assert regenerated_code[10].strip() == "END FORALL"
+
+
+@pytest.mark.parametrize('frontend', available_frontends(
+    xfail=[(OMNI, 'No support for Cray Pointers')]
+))
+def test_cray_pointers(frontend):
+    fcode = """
+SUBROUTINE SUBROUTINE_WITH_CRAY_POINTER (KLON,KLEV,POOL)
+IMPLICIT NONE
+INTEGER, INTENT(IN) :: KLON, KLEV
+REAL, INTENT(INOUT) :: POOL(:)
+REAL, DIMENSION(KLON,KLEV) :: ZQ
+POINTER(IP_ZQ, ZQ)
+IP_ZQ = LOC(POOL)
+END SUBROUTINE
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    intrinsics = FindNodes(ir.Intrinsic).visit(routine.spec)
+    assert len(intrinsics) == 2
+    assert 'IMPLICIT NONE' in intrinsics[0].text
+    assert 'POINTER(IP_ZQ, ZQ)' in intrinsics[1].text
+    assert 'POINTER(IP_ZQ, ZQ)' in routine.to_fortran()
