@@ -5,29 +5,26 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
-from pathlib import Path
+import platform
 import pytest
 import numpy as np
 
 from loki import Module, Subroutine, fgen
 from loki.build import jit_compile, jit_compile_lib, clean_test, Builder, Obj
 from loki.expression import symbols as sym, FindVariables
-from loki.frontend import available_frontends
-from loki.ir import FindNodes, CallStatement
+from loki.frontend import available_frontends, OMNI
+from loki.ir import FindNodes, CallStatement, Loop
 
 from loki.transformations.array_indexing import (
-    promote_variables, demote_variables, normalize_range_indexing,
-    invert_array_indices, flatten_arrays,
-    normalize_array_shape_and_access, shift_to_zero_indexing,
+    promote_variables, demote_variables, invert_array_indices,
+    flatten_arrays, normalize_array_shape_and_access,
+    shift_to_zero_indexing, resolve_vector_notation,
+    LowerConstantArrayIndices, remove_explicit_array_dimensions,
+    add_explicit_array_dimensions
 )
 from loki.transformations.transpile import FortranCTransformation
 
 
-@pytest.fixture(scope='module', name='here')
-def fixture_here():
-    return Path(__file__).parent
-
-
 @pytest.fixture(scope='function', name='builder')
 def fixture_builder(tmp_path):
     yield Builder(source_dirs=tmp_path, build_dir=tmp_path)
@@ -35,7 +32,7 @@ def fixture_builder(tmp_path):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_promote_variable_scalar(here, frontend):
+def test_transform_promote_variable_scalar(tmp_path, frontend):
     """
     Apply variable promotion for a single scalar variable.
     """
@@ -55,7 +52,7 @@ end subroutine transform_promote_variable_scalar
     routine = Subroutine.from_source(fcode, frontend=frontend)
 
     # Test the original implementation
-    filepath = here/(f'{routine.name}_{frontend}.f90')
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
     ret = function()
     assert ret == 55
@@ -66,17 +63,14 @@ end subroutine transform_promote_variable_scalar
     assert isinstance(routine.variable_map['tmp'], sym.Array)
     assert routine.variable_map['tmp'].shape == (sym.Literal(10),)
 
-    promoted_filepath = here/(f'{routine.name}_promoted_{frontend}.f90')
+    promoted_filepath = tmp_path/(f'{routine.name}_promoted_{frontend}.f90')
     promoted_function = jit_compile(routine, filepath=promoted_filepath, objname=routine.name)
     ret = promoted_function()
     assert ret == 55
 
-    clean_test(filepath)
-    clean_test(promoted_filepath)
-
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_promote_variables(here, frontend):
+def test_transform_promote_variables(tmp_path, frontend):
     """
     Apply variable promotion for scalar and array variables.
     """
@@ -110,10 +104,9 @@ subroutine transform_promote_variables(scalar, vector, n)
 end subroutine transform_promote_variables
     """.strip()
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
 
     # Test the original implementation
-    filepath = here/(f'{routine.name}_{frontend}.f90')
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
 
     n = 10
@@ -153,7 +146,7 @@ end subroutine transform_promote_variables
     assert routine.variable_map['tmp_matrix'].shape == (routine.variable_map['n'], ) * 3
 
     # Test promoted routine
-    promoted_filepath = here/(f'{routine.name}_promoted_{frontend}.f90')
+    promoted_filepath = tmp_path/(f'{routine.name}_promoted_{frontend}.f90')
     promoted_function = jit_compile(routine, filepath=promoted_filepath, objname=routine.name)
 
     scalar = np.zeros(shape=(1,), order='F', dtype=np.int32)
@@ -163,12 +156,9 @@ end subroutine transform_promote_variables
     assert np.all(vector[:-1] == np.array(list(range(n + 1, 2*n)), order='F', dtype=np.int32))
     assert vector[-1] == 3*n
 
-    clean_test(filepath)
-    clean_test(promoted_filepath)
-
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_demote_variables(here, frontend):
+def test_transform_demote_variables(tmp_path, frontend):
     """
     Apply variable demotion to a range of array variables.
     """
@@ -207,10 +197,9 @@ subroutine transform_demote_variables(scalar, vector, matrix, n, m)
 end subroutine transform_demote_variables
     """.strip()
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
 
     # Test the original implementation
-    filepath = here/(f'{routine.name}_{frontend}.f90')
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
 
     n = 3
@@ -238,7 +227,7 @@ end subroutine transform_demote_variables
     assert routine.variable_map['tmp_matrix'].shape == (routine.variable_map['n'], routine.variable_map['n'])
 
     # Test promoted routine
-    demoted_filepath = here/(f'{routine.name}_demoted_{frontend}.f90')
+    demoted_filepath = tmp_path/(f'{routine.name}_demoted_{frontend}.f90')
     demoted_function = jit_compile(routine, filepath=demoted_filepath, objname=routine.name)
 
     n = 3
@@ -260,33 +249,32 @@ end subroutine transform_demote_variables
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_demote_dimension_arguments(here, frontend):
+def test_transform_demote_dimension_arguments(tmp_path, frontend):
     """
     Apply variable demotion to array arguments defined with DIMENSION
     keywords.
     """
     fcode = """
 subroutine transform_demote_dimension_arguments(vec1, vec2, matrix, n, m)
-  implicit none
-  integer, intent(in) :: n, m
-  integer, dimension(n), intent(inout) :: vec1, vec2
-  integer, dimension(n, m), intent(inout) :: matrix
-  integer, dimension(n) :: vec_tmp
-  integer :: i, j
+    implicit none
+    integer, intent(in) :: n, m
+    integer, dimension(n), intent(inout) :: vec1, vec2
+    integer, dimension(n, m), intent(inout) :: matrix
+    integer, dimension(n) :: vec_tmp
+    integer :: i, j
 
-  do i=1,n
-    do j=1,m
-      vec_tmp(i) = vec1(i) + vec2(i)
-      matrix(i, j) = matrix(i, j) + vec_tmp(i)
+    do i=1,n
+        do j=1,m
+        vec_tmp(i) = vec1(i) + vec2(i)
+        matrix(i, j) = matrix(i, j) + vec_tmp(i)
+        end do
     end do
-  end do
 end subroutine transform_demote_dimension_arguments
 """
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
 
     # Test the original implementation
-    filepath = here/(f'{routine.name}_{frontend}.f90')
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
 
     assert isinstance(routine.variable_map['vec1'], sym.Array)
@@ -316,7 +304,7 @@ end subroutine transform_demote_dimension_arguments
     assert routine.variable_map['matrix'].shape == (routine.variable_map['m'],)
 
     # Test promoted routine
-    demoted_filepath = here/(f'{routine.name}_demoted_{frontend}.f90')
+    demoted_filepath = tmp_path/(f'{routine.name}_demoted_{frontend}.f90')
     demoted_function = jit_compile(routine, filepath=demoted_filepath, objname=routine.name)
 
     n = 3
@@ -330,9 +318,11 @@ end subroutine transform_demote_dimension_arguments
     assert np.all(vec2 == 2) and np.sum(vec2) == 6
     assert np.all(matrix == 16) and np.sum(matrix) == 32
 
+
+@pytest.mark.skipif(platform.system() == 'Darwin', reason='Unclear issue causing problems on MacOS (#352)')
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('start_index', (0, 1, 5))
-def test_transform_normalize_array_shape_and_access(here, frontend, start_index):
+def test_transform_normalize_array_shape_and_access(tmp_path, frontend, start_index):
     """
     Test normalization of array shape and access, thus changing arrays with start
     index different than "1" to have start index "1".
@@ -416,11 +406,9 @@ def test_transform_normalize_array_shape_and_access(here, frontend, start_index)
     l2 = 3
     l3 = 4
     l4 = 5
-    module = Module.from_source(fcode, frontend=frontend)
-    for routine in module.routines:
-        normalize_range_indexing(routine) # Fix OMNI nonsense
-    filepath = here/(f'norm_arr_shape_access_{frontend}.f90')
-    # compile and test "original" module/function
+    module = Module.from_source(fcode, frontend=frontend, xmods=[tmp_path])
+    filepath = tmp_path/(f'norm_arr_shape_access_{frontend}.f90')
+    # compile and test "original" module/function
     mod = jit_compile(module, filepath=filepath, objname='norm_arr_shape_access_mod')
     function = getattr(mod, 'norm_arr_shape_access')
     orig_x1, orig_x2, orig_x3, orig_x4, orig_assumed_x1 = init_arguments(l1, l2, l3, l4)
@@ -431,7 +419,7 @@ def test_transform_normalize_array_shape_and_access(here, frontend, start_index)
     for routine in module.routines:
         normalize_array_shape_and_access(routine)
 
-    filepath = here/(f'norm_arr_shape_access_normalized_{frontend}.f90')
+    filepath = tmp_path/(f'norm_arr_shape_access_normalized_{frontend}.f90')
     # compile and test "normalized" module/function
     mod = jit_compile(module, filepath=filepath, objname='norm_arr_shape_access_mod')
     function = getattr(mod, 'norm_arr_shape_access')
@@ -453,7 +441,7 @@ def test_transform_normalize_array_shape_and_access(here, frontend, start_index)
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('start_index', (0, 1, 5))
-def test_transform_flatten_arrays(here, frontend, builder, start_index):
+def test_transform_flatten_arrays(tmp_path, frontend, builder, start_index):
     """
     Test flattening or arrays, meaning converting multi-dimensional
     arrays to one-dimensional arrays including corresponding
@@ -513,8 +501,7 @@ def test_transform_flatten_arrays(here, frontend, builder, start_index):
     l4 = 5
     # Test the original implementation
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
-    filepath = here/(f'{routine.name}_{start_index}_{frontend}.f90')
+    filepath = tmp_path/(f'{routine.name}_{start_index}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
     orig_x1, orig_x2, orig_x3, orig_x4 = init_arguments(l1, l2, l3, l4)
     function(orig_x1, orig_x2, orig_x3, orig_x4, l1, l2, l3, l4)
@@ -523,9 +510,8 @@ def test_transform_flatten_arrays(here, frontend, builder, start_index):
     # Test flattening order='F'
     f_routine = Subroutine.from_source(fcode, frontend=frontend)
     normalize_array_shape_and_access(f_routine)
-    normalize_range_indexing(f_routine) # Fix OMNI nonsense
     flatten_arrays(routine=f_routine, order='F', start_index=1)
-    filepath = here/(f'{f_routine.name}_{start_index}_flattened_F_{frontend}.f90')
+    filepath = tmp_path/(f'{f_routine.name}_{start_index}_flattened_F_{frontend}.f90')
     function = jit_compile(f_routine, filepath=filepath, objname=routine.name)
     f_x1, f_x2, f_x3, f_x4 = init_arguments(l1, l2, l3, l4, flattened=True)
     function(f_x1, f_x2, f_x3, f_x4, l1, l2, l3, l4)
@@ -540,10 +526,9 @@ def test_transform_flatten_arrays(here, frontend, builder, start_index):
     # Test flattening order='C'
     c_routine = Subroutine.from_source(fcode, frontend=frontend)
     normalize_array_shape_and_access(c_routine)
-    normalize_range_indexing(c_routine) # Fix OMNI nonsense
     invert_array_indices(c_routine)
     flatten_arrays(routine=c_routine, order='C', start_index=1)
-    filepath = here/(f'{c_routine.name}_{start_index}_flattened_C_{frontend}.f90')
+    filepath = tmp_path/(f'{c_routine.name}_{start_index}_flattened_C_{frontend}.f90')
     function = jit_compile(c_routine, filepath=filepath, objname=routine.name)
     c_x1, c_x2, c_x3, c_x4 = init_arguments(l1, l2, l3, l4, flattened=True)
     function(c_x1, c_x2, c_x3, c_x4, l1, l2, l3, l4)
@@ -560,9 +545,9 @@ def test_transform_flatten_arrays(here, frontend, builder, start_index):
     # Test C transpilation (which includes flattening)
     f2c_routine = Subroutine.from_source(fcode, frontend=frontend)
     f2c = FortranCTransformation()
-    f2c.apply(source=f2c_routine, path=here)
+    f2c.apply(source=f2c_routine, path=tmp_path)
     libname = f'fc_{f2c_routine.name}_{start_index}_{frontend}'
-    c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=here, name=libname, builder=builder)
+    c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=tmp_path, name=libname, builder=builder)
     fc_function = c_kernel.transf_flatten_arr_fc_mod.transf_flatten_arr_fc
     f2c_x1, f2c_x2, f2c_x3, f2c_x4 = init_arguments(l1, l2, l3, l4, flattened=True)
     fc_function(f2c_x1, f2c_x2, f2c_x3, f2c_x4, l1, l2, l3, l4)
@@ -573,11 +558,6 @@ def test_transform_flatten_arrays(here, frontend, builder, start_index):
     assert (f2c_x3 == orig_x3.flatten(order='F')).all()
     assert (f2c_x4 == orig_x4.flatten(order='F')).all()
 
-    builder.clean()
-    clean_test(filepath)
-    f2c.wrapperpath.unlink()
-    f2c.c_path.unlink()
-
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('ignore', ((), ('i2',), ('i4', 'i1')))
@@ -646,7 +626,7 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
     """
     array_dims = '(:,:)' if explicit_dimensions else ''
     fcode_driver = f"""
-  SUBROUTINE driver_routine(nlon, nlev, a, b)
+SUBROUTINE driver_routine(nlon, nlev, a, b)
     use kernel_mod, only: kernel_routine
     INTEGER, INTENT(IN)    :: nlon, nlev
     INTEGER, INTENT(INOUT) :: a(nlon,nlev)
@@ -654,13 +634,13 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
 
     call kernel_routine(nlon, nlev, a{array_dims}, b{array_dims})
 
-  END SUBROUTINE driver_routine
+END SUBROUTINE driver_routine
     """
     fcode_kernel = """
-  module kernel_mod
-  IMPLICIT NONE
-  CONTAINS
-  SUBROUTINE kernel_routine(nlon, nlev, a, b)
+module kernel_mod
+IMPLICIT NONE
+CONTAINS
+SUBROUTINE kernel_routine(nlon, nlev, a, b)
     INTEGER, INTENT(IN)    :: nlon, nlev
     INTEGER, INTENT(INOUT) :: a(nlon,nlev)
     INTEGER, INTENT(INOUT) :: b(nlon,nlev)
@@ -672,8 +652,8 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
         b(i,j) = i*10 + j + 1
       end do
     end do
-  END SUBROUTINE kernel_routine
-  end module kernel_mod
+END SUBROUTINE kernel_routine
+end module kernel_mod
     """
     def init_arguments(nlon, nlev, flattened=False):
         a = np.zeros(shape=(nlon*nlev) if flattened else (nlon,nlev,), order='F', dtype=np.int32)
@@ -685,8 +665,9 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
         assert all(len(arr.dimensions) == 1 or not arr.dimensions for arr in arrays)
         assert all(len(arr.shape) == 1 for arr in arrays)
 
-    kernel_module = Module.from_source(fcode_kernel, frontend=frontend)
-    driver = Subroutine.from_source(fcode_driver, frontend=frontend)
+    kernel_module = Module.from_source(fcode_kernel, frontend=frontend, xmods=[tmp_path])
+    driver = Subroutine.from_source(fcode_driver, frontend=frontend, xmods=[tmp_path],
+            definitions=kernel_module)
     kernel = kernel_module.subroutines[0]
 
     # check for a(:,:) and b(:,:) if "explicit_dimensions"
@@ -709,9 +690,6 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
     ref_function(nlon, nlev, a_ref, b_ref)
     builder.clean()
 
-    normalize_range_indexing(driver)
-    normalize_range_indexing(kernel)
-
     # flatten all the arrays in the kernel and driver
     flatten_arrays(routine=kernel, order='F', start_index=1)
     flatten_arrays(routine=driver, order='F', start_index=1)
@@ -733,3 +711,439 @@ def test_transform_flatten_arrays_call(tmp_path, frontend, builder, explicit_dim
     assert (b_flattened == b_ref.flatten(order='F')).all()
 
     builder.clean()
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('recurse_to_kernels', (False, True))
+@pytest.mark.parametrize('inline_external_only', (False, True))
+@pytest.mark.parametrize('pass_as_kwarg', (False, True,))
+def test_lower_constant_array_indices(tmp_path, frontend, recurse_to_kernels, inline_external_only, pass_as_kwarg):
+    """
+    Test lowering constant array indices
+    """
+    fcode_driver = f"""
+subroutine driver(nlon,nlev,nb,var)
+  use kernel_mod, only: kernel
+  implicit none
+  integer, parameter :: param_1 = 1
+  integer, parameter :: param_2 = 2
+  integer, parameter :: param_3 = 5
+  integer, intent(in) :: nlon,nlev,nb
+  real, intent(inout) :: var(nlon,nlev,param_3,nb)
+  integer :: ibl
+  integer :: offset
+  integer :: some_val
+  integer :: loop_start, loop_end
+  loop_start = 2
+  loop_end = nb
+  some_val = 0
+  offset = 1
+  !$omp test
+  do ibl=loop_start, loop_end
+    call kernel(nlon,nlev,{'var=' if pass_as_kwarg else ''}var(:,:,param_1,ibl), {'another_var=' if pass_as_kwarg else ''}var(:,:,param_2:param_3,ibl), {'icend=' if pass_as_kwarg else ''}offset, {'lstart=' if pass_as_kwarg else ''}loop_start, {'lend=' if pass_as_kwarg else ''}loop_end)
+    call kernel(nlon,nlev,{'var=' if pass_as_kwarg else ''}var(:,:,param_1,ibl), {'another_var=' if pass_as_kwarg else ''}var(:,:,param_2:param_3,ibl), {'icend=' if pass_as_kwarg else ''}offset, {'lstart=' if pass_as_kwarg else ''}loop_start, {'lend=' if pass_as_kwarg else ''}loop_end)
+    ! call kernel(nlon,nlev,var(:,:,param_1,ibl), var(:,:,param_2:param_3,ibl), offset, loop_start, loop_end)
+  enddo
+end subroutine driver
+"""
+
+    fcode_kernel = """
+module kernel_mod
+implicit none
+contains
+subroutine kernel(nlon,nlev,var,another_var,icend,lstart,lend)
+  use compute_mod, only: compute
+  implicit none
+  integer, intent(in) :: nlon,nlev,icend,lstart,lend
+  real, intent(inout) :: var(nlon,nlev)
+  real, intent(inout) :: another_var(nlon,nlev,4)
+  integer :: jk, jl, jt
+  var(:,:) = 0.
+  do jk = 1,nlev
+    do jl = 1, nlon
+      var(jl, jk) = 0.
+      do jt= 1,4
+        another_var(jl, jk, jt) = 0.0
+      end do
+    end do
+  end do
+  call compute(nlon,nlev,var)
+  call compute(nlon,nlev,var)
+end subroutine kernel
+end module kernel_mod
+"""
+
+    fcode_nested_kernel = """
+module compute_mod
+implicit none
+contains
+subroutine compute(nlon,nlev,var)
+  implicit none
+  integer, intent(in) :: nlon,nlev
+  real, intent(inout) :: var(nlon,nlev)
+  var(:,:) = 0.
+end subroutine compute
+end module compute_mod
+"""
+
+    nested_kernel_mod = Module.from_source(fcode_nested_kernel, frontend=frontend, xmods=[tmp_path])
+    kernel_mod = Module.from_source(fcode_kernel, frontend=frontend, definitions=nested_kernel_mod, xmods=[tmp_path])
+    driver = Subroutine.from_source(fcode_driver, frontend=frontend, definitions=kernel_mod, xmods=[tmp_path])
+
+    kwargs = {'recurse_to_kernels': recurse_to_kernels, 'inline_external_only': inline_external_only}
+    LowerConstantArrayIndices(**kwargs).apply(driver, role='driver', targets=('kernel',))
+    LowerConstantArrayIndices(**kwargs).apply(kernel_mod['kernel'], role='kernel', targets=('compute',))
+    LowerConstantArrayIndices(**kwargs).apply(nested_kernel_mod['compute'], role='kernel')
+
+    # driver
+    kernel_calls = FindNodes(CallStatement).visit(driver.body)
+    for kernel_call in kernel_calls:
+        if pass_as_kwarg:
+            arg1 = kernel_call.kwarguments[0][1]
+            arg2 = kernel_call.kwarguments[1][1]
+        else:
+            arg1 = kernel_call.arguments[2]
+            arg2 = kernel_call.arguments[3]
+        if inline_external_only and frontend != OMNI:
+            assert arg1.dimensions == (':', ':', 'param_1', 'ibl')
+            assert arg2.dimensions == (':', ':', 'param_2:param_3', 'ibl')
+        else:
+            assert arg1.dimensions == (':', ':', ':', 'ibl')
+            assert arg2.dimensions == (':', ':', ':', 'ibl')
+    # kernel
+    kernel_vars = kernel_mod['kernel'].variable_map
+    if inline_external_only and frontend != OMNI:
+        assert kernel_vars['var'].shape == ('nlon', 'nlev')
+        assert kernel_vars['var'].dimensions == ('nlon', 'nlev')
+        assert kernel_vars['another_var'].shape == ('nlon', 'nlev', 4)
+        assert kernel_vars['another_var'].dimensions == ('nlon', 'nlev', 4)
+    else:
+        assert kernel_vars['var'].shape == ('nlon', 'nlev', 5)
+        assert kernel_vars['var'].dimensions == ('nlon', 'nlev', 5)
+        assert kernel_vars['another_var'].shape == ('nlon', 'nlev', 5)
+        assert kernel_vars['another_var'].dimensions == ('nlon', 'nlev', 5)
+    if inline_external_only and frontend != OMNI:
+        for var in FindVariables().visit(kernel_mod['kernel'].body):
+            if var.name.lower() == 'var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert var.dimensions == ('jl', 'jk')
+            if var.name.lower() == 'another_var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert tuple(str(dim) for dim in var.dimensions) == ('jl', 'jk', 'jt')
+    else:
+        for var in FindVariables().visit(kernel_mod['kernel'].body):
+            if var.name.lower() == 'var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert var.dimensions == ('jl', 'jk', 1)
+            if var.name.lower() == 'another_var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert tuple(str(dim) for dim in var.dimensions) == ('jl', 'jk', 'jt + 2 + -1')
+    compute_calls = FindNodes(CallStatement).visit(kernel_mod['kernel'].body)
+    for compute_call in compute_calls:
+        for arg in compute_call.arguments:
+            if arg.name.lower() == 'var':
+                if inline_external_only and frontend != OMNI:
+                    assert arg.dimensions == (':', ':')
+                elif recurse_to_kernels:
+                    assert arg.dimensions == (':', ':', ':')
+                else:
+                    assert arg.dimensions == (':', ':', '1')
+    # nested kernel
+    nested_kernel_var = nested_kernel_mod['compute'].variable_map['var']
+    if recurse_to_kernels and (not inline_external_only or frontend == OMNI):
+        assert nested_kernel_var.shape == ('nlon', 'nlev', 5)
+        assert nested_kernel_var.dimensions == ('nlon', 'nlev', 5)
+        for var in FindVariables().visit(nested_kernel_mod['compute'].body):
+            if var.name.lower() == 'var':
+                assert var.dimensions == (':', ':', 1)
+    else:
+        assert nested_kernel_var.shape == ('nlon', 'nlev')
+        assert nested_kernel_var.dimensions == ('nlon', 'nlev')
+        for var in FindVariables().visit(nested_kernel_mod['compute'].body):
+            if var.name.lower() == 'var':
+                assert var.dimensions == (':', ':')
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('recurse_to_kernels', (False, True,))
+@pytest.mark.parametrize('inline_external_only', (False, True,))
+def test_lower_constant_array_indices_academic(tmp_path, frontend, recurse_to_kernels, inline_external_only):
+    """
+    Test lowering constant array indices for a valid but somewhat academic example ...
+
+    The transformation is capable to handle that, but let's just hope we'll never see
+    something like that out there in the wild ...
+    """
+    fcode_driver = """
+subroutine driver(nlon,nlev,nb,var)
+  use kernel_mod, only: kernel
+  implicit none
+  integer, parameter :: param_1 = 1
+  integer, parameter :: param_2 = 2
+  integer, parameter :: param_3 = 5
+  integer, intent(in) :: nlon,nlev,nb
+  real, intent(inout) :: var(nlon,4,3,nlev,param_3,nb)
+  ! real, intent(inout) :: var(nlon,3,nlev,param_3,nb)
+  integer :: ibl, j
+  integer :: offset
+  integer :: some_val
+  integer :: loop_start, loop_end
+  loop_start = 2
+  loop_end = nb
+  some_val = 0
+  offset = 1
+  !$omp test
+  do ibl=loop_start, loop_end
+    do j=1,4
+      call kernel(nlon,nlev,var(:,j,1,:,param_1,ibl), var(:,j,2:3,:,param_2:param_3,ibl), offset, loop_start, loop_end)
+      call kernel(nlon,nlev,var(:,j,1,:,param_1,ibl), var(:,j,2:3,:,param_2:param_3,ibl), offset, loop_start, loop_end)
+    end do
+  enddo
+end subroutine driver
+"""
+
+    fcode_kernel = """
+module kernel_mod
+implicit none
+contains
+subroutine kernel(nlon,nlev,var,another_var,icend,lstart,lend)
+  use compute_mod, only: compute
+  implicit none
+  integer, intent(in) :: nlon,nlev,icend,lstart,lend
+  real, intent(inout) :: var(nlon,nlev)
+  real, intent(inout) :: another_var(nlon,2,nlev,4)
+  integer :: jk, jl, jt
+  var(:,:) = 0.
+  do jk = 1,nlev
+    do jl = 1, nlon
+      var(jl, jk) = 0.
+      do jt= 1,4
+        another_var(jl, 1, jk, jt) = 0.0
+      end do
+    end do
+  end do
+  call compute(nlon,nlev,var)
+  call compute(nlon,nlev,var)
+end subroutine kernel
+end module kernel_mod
+"""
+
+    fcode_nested_kernel = """
+module compute_mod
+implicit none
+contains
+subroutine compute(nlon,nlev,var)
+  implicit none
+  integer, intent(in) :: nlon,nlev
+  real, intent(inout) :: var(nlon,nlev)
+  var(:,:) = 0.
+end subroutine compute
+end module compute_mod
+"""
+
+    nested_kernel_mod = Module.from_source(fcode_nested_kernel, frontend=frontend, xmods=[tmp_path])
+    kernel_mod = Module.from_source(fcode_kernel, frontend=frontend, definitions=nested_kernel_mod, xmods=[tmp_path])
+    driver = Subroutine.from_source(fcode_driver, frontend=frontend, definitions=kernel_mod, xmods=[tmp_path])
+
+    kwargs = {'recurse_to_kernels': recurse_to_kernels, 'inline_external_only': inline_external_only}
+    LowerConstantArrayIndices(**kwargs).apply(driver, role='driver', targets=('kernel',))
+    LowerConstantArrayIndices(**kwargs).apply(kernel_mod['kernel'], role='kernel', targets=('compute',))
+    LowerConstantArrayIndices(**kwargs).apply(nested_kernel_mod['compute'], role='kernel')
+
+    # driver
+    kernel_calls = FindNodes(CallStatement).visit(driver.body)
+    for kernel_call in kernel_calls:
+        if inline_external_only and frontend != OMNI:
+            assert kernel_call.arguments[2].dimensions == (':', 'j', ':', ':', 'param_1', 'ibl')
+            assert kernel_call.arguments[3].dimensions == (':', 'j', ':', ':', 'param_2:param_3', 'ibl')
+        else:
+            assert kernel_call.arguments[2].dimensions == (':', 'j', ':', ':', ':', 'ibl')
+            assert kernel_call.arguments[3].dimensions == (':', 'j', ':', ':', ':', 'ibl')
+    # kernel
+    kernel_vars = kernel_mod['kernel'].variable_map
+    if inline_external_only and frontend != OMNI:
+        assert kernel_vars['var'].shape == ('nlon', 3, 'nlev')
+        assert kernel_vars['var'].dimensions == ('nlon', 3, 'nlev')
+        assert kernel_vars['another_var'].shape == ('nlon', 3, 'nlev', 4)
+        assert kernel_vars['another_var'].dimensions == ('nlon', 3, 'nlev', 4)
+    else:
+        assert kernel_vars['var'].shape == ('nlon', '3', 'nlev', 5)
+        assert kernel_vars['var'].dimensions == ('nlon', '3', 'nlev', 5)
+        assert kernel_vars['another_var'].shape == ('nlon', '3', 'nlev', 5)
+        assert kernel_vars['another_var'].dimensions == ('nlon', '3', 'nlev', 5)
+    if inline_external_only and frontend != OMNI:
+        for var in FindVariables().visit(kernel_mod['kernel'].body):
+            if var.name.lower() == 'var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert var.dimensions == ('jl', 1, 'jk')
+            if var.name.lower() == 'another_var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert tuple(str(dim) for dim in var.dimensions) == ('jl', '1 + 2 + -1', 'jk', 'jt')
+    else:
+        for var in FindVariables().visit(kernel_mod['kernel'].body):
+            if var.name.lower() == 'var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert var.dimensions == ('jl', 1, 'jk', 1)
+            if var.name.lower() == 'another_var' and not any(isinstance(dim, sym.RangeIndex) for dim in var.dimensions):
+                assert tuple(str(dim) for dim in var.dimensions) == ('jl', '1 + 2 + -1', 'jk', 'jt + 2 + -1')
+    compute_calls = FindNodes(CallStatement).visit(kernel_mod['kernel'].body)
+    for compute_call in compute_calls:
+        for arg in compute_call.arguments:
+            if arg.name.lower() == 'var':
+                if inline_external_only and frontend != OMNI:
+                    if recurse_to_kernels:
+                        assert arg.dimensions == (':', ':', ':')
+                    else:
+                        assert arg.dimensions == (':', 1, ':')
+                elif recurse_to_kernels:
+                    assert arg.dimensions == (':', ':', ':', ':')
+                else:
+                    assert arg.dimensions == (':', 1, ':', '1')
+    # nested kernel
+    nested_kernel_var = nested_kernel_mod['compute'].variable_map['var']
+    if recurse_to_kernels and (not inline_external_only or frontend == OMNI):
+        assert nested_kernel_var.shape == ('nlon', 3, 'nlev', 5)
+        assert nested_kernel_var.dimensions == ('nlon', 3, 'nlev', 5)
+        for var in FindVariables().visit(nested_kernel_mod['compute'].body):
+            if var.name.lower() == 'var':
+                assert var.dimensions == (':', 1, ':', 1)
+    else:
+        if recurse_to_kernels:
+            assert nested_kernel_var.shape == ('nlon', 3, 'nlev')
+            assert nested_kernel_var.dimensions == ('nlon', 3, 'nlev')
+        else:
+            assert nested_kernel_var.shape == ('nlon', 'nlev')
+            assert nested_kernel_var.dimensions == ('nlon', 'nlev')
+        for var in FindVariables().visit(nested_kernel_mod['compute'].body):
+            if var.name.lower() == 'var':
+                if recurse_to_kernels:
+                    assert var.dimensions == (':', 1, ':')
+                else:
+                    assert var.dimensions == (':', ':')
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_transform_promote_resolve_vector_notation(tmp_path, frontend):
+    """
+    Apply and test resolve vector notation utility.
+    """
+    fcode = """
+subroutine transform_resolve_vector_notation(ret1, ret2)
+  implicit none
+  integer, parameter :: param1 = 3
+  integer, parameter :: param2 = 5
+  integer, intent(out) :: ret1(param1, param1), ret2(param1, param2)
+  integer :: tmp, jk
+
+  ret1(:, :) = 11
+  ret2(:, :) = 42
+
+end subroutine transform_resolve_vector_notation
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    resolve_vector_notation(routine)
+
+    loops = FindNodes(Loop).visit(routine.body)
+    arrays = [var for var in FindVariables(unique=False).visit(routine.body) if isinstance(var, sym.Array)]
+
+    assert len(loops) == 4
+    assert loops[0].variable == 'i_ret1_1'
+    assert loops[0].bounds.children == (1, 'param1', 1) if frontend != OMNI else (1, 3, 1)
+    assert loops[1].variable == 'i_ret1_0'
+    assert loops[1].bounds.children == (1, 'param1', 1) if frontend != OMNI else (1, 3, 1)
+    assert loops[2].variable == 'i_ret2_1'
+    assert loops[2].bounds.children == (1, 'param2', 1) if frontend != OMNI else (1, 5, 1)
+    assert loops[3].variable == 'i_ret2_0'
+    assert loops[3].bounds.children == (1, 'param1', 1) if frontend != OMNI else (1, 3, 1)
+
+    assert len(arrays) == 2
+    assert arrays[0].dimensions == ('i_ret1_0', 'i_ret1_1')
+    assert arrays[1].dimensions == ('i_ret2_0', 'i_ret2_1')
+
+    ret1 = np.zeros(shape=(3, 3), order='F', dtype=np.int32)
+    ret2 = np.zeros(shape=(3, 5), order='F', dtype=np.int32)
+
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
+    function = jit_compile(routine, filepath=filepath, objname=routine.name)
+    function(ret1, ret2)
+
+    assert np.all(ret1 == 11)
+    assert np.all(ret2 == 42)
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('calls_only', (False, True))
+def test_transform_explicit_dimensions(tmp_path, frontend, builder, calls_only):
+    """
+    Test flattening or arrays, meaning converting multi-dimensional
+    arrays to one-dimensional arrays including corresponding
+    index arithmetic (for calls).
+    """
+    fcode_driver = """
+  SUBROUTINE driver_routine(nlon, nlev, a, b)
+    use kernel_mod, only: kernel_routine
+    INTEGER, INTENT(IN)    :: nlon, nlev
+    INTEGER, INTENT(INOUT) :: a(nlon,nlev)
+    INTEGER, INTENT(INOUT)  :: b(nlon,nlev)
+
+    call kernel_routine(nlon, nlev, a, b)
+
+  END SUBROUTINE driver_routine
+    """
+    fcode_kernel = """
+  module kernel_mod
+  IMPLICIT NONE
+  CONTAINS
+  SUBROUTINE kernel_routine(nlon, nlev, a, b)
+    INTEGER, INTENT(IN)    :: nlon, nlev
+    INTEGER, INTENT(INOUT) :: a(nlon,nlev)
+    INTEGER, INTENT(INOUT) :: b(nlon,nlev)
+
+    a = a + b
+  END SUBROUTINE kernel_routine
+  end module kernel_mod
+    """
+    def init_arguments(nlon, nlev):
+        a = 2*np.ones(shape=(nlon,nlev,), order='F', dtype=np.int32)
+        b = 3*np.ones(shape=(nlon,nlev,), order='F', dtype=np.int32)
+        return a, b
+
+    kernel_module = Module.from_source(fcode_kernel, frontend=frontend, xmods=[tmp_path])
+    driver = Subroutine.from_source(fcode_driver, frontend=frontend, xmods=[tmp_path],
+            definitions=[kernel_module])
+    kernel = kernel_module.subroutines[0]
+
+    # compile and test reference
+    refname = f'ref_explicit_dims_{driver.name}_{frontend}'
+    reference = jit_compile_lib([kernel_module, driver], path=tmp_path, name=refname, builder=builder)
+    ref_function = reference.driver_routine
+
+    nlon = 10
+    nlev = 12
+    a_ref, b_ref = init_arguments(nlon, nlev)
+    ref_function(nlon, nlev, a_ref, b_ref)
+    builder.clean()
+
+    # add explicit array dimensions
+    add_explicit_array_dimensions(driver)
+    add_explicit_array_dimensions(kernel)
+    kernel_call = FindNodes(CallStatement).visit(driver.body)[0]
+    kernel_call_array_args = [arg for arg in kernel_call.arguments if isinstance(arg, sym.Array)]
+    assert all(len(arg.dimensions) == 2 for arg in kernel_call_array_args)
+
+    # remove explicit array dimensions (possibly only for calls)
+    remove_explicit_array_dimensions(driver, calls_only=calls_only)
+    remove_explicit_array_dimensions(kernel, calls_only=calls_only)
+    kernel_call = FindNodes(CallStatement).visit(driver.body)[0]
+    kernel_call_array_args = [arg for arg in kernel_call.arguments if isinstance(arg, sym.Array)]
+    assert all(not arg.dimensions for arg in kernel_call_array_args)
+    kernel_arrays = FindVariables().visit(kernel.body)
+    if calls_only:
+        assert all(len(arr.dimensions) == 2 for arr in kernel_arrays)
+    else:
+        assert all(not arr.dimensions for arr in kernel_arrays)
+
+    # compile and test the resulting code
+    testname = f'test_explicit_dims_{"calls_only_" if calls_only else ""}_{driver.name}_{frontend}'
+    test = jit_compile_lib([kernel_module, driver], path=tmp_path, name=testname, builder=builder)
+    test_function = test.driver_routine
+
+    a_test, b_test = init_arguments(nlon, nlev)
+    test_function(nlon, nlev, a_test, b_test)
+
+    # check whether reference and flattened variant(s) produce same result
+    assert (a_test == a_ref).all()
+    assert (b_test == b_ref).all()
+
+    builder.clean()
