@@ -14,7 +14,7 @@ from loki import (
     ProcedureDeclaration, BasicType, CaseInsensitiveDict,
 )
 from loki.expression import Scalar, Array, FindVariables, FindInlineCalls
-from loki.frontend import available_frontends, OMNI, OFP
+from loki.frontend import available_frontends, OMNI
 from loki.ir import FindNodes, CallStatement
 
 from loki.transformations.transform_derived_types import (
@@ -52,7 +52,7 @@ def fixture_config():
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_analysis(frontend):
+def test_transform_derived_type_arguments_analysis(frontend, tmp_path):
     fcode = f"""
 module transform_derived_type_arguments_mod
 
@@ -86,7 +86,7 @@ contains
 end module transform_derived_type_arguments_mod
     """.strip()
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     item = ProcedureItem(name='transform_derived_type_arguments_mod#kernel', source=source)
     source['kernel'].apply(DerivedTypeArgumentsTransformation(), role='kernel', item=item)
 
@@ -110,7 +110,7 @@ end module transform_derived_type_arguments_mod
 
 @pytest.mark.parametrize('all_derived_types', (False, True))
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_expansion_trivial_derived_type(frontend, all_derived_types):
+def test_transform_derived_type_arguments_expansion_trivial_derived_type(frontend, all_derived_types, tmp_path):
     fcode = """
 module transform_derived_type_arguments_mod
 
@@ -153,7 +153,7 @@ contains
 end module transform_derived_type_arguments_mod
     """.strip()
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     call_tree = [
         ProcedureItem(name='transform_derived_type_arguments_mod#caller', source=source, config={'role': 'driver'}),
@@ -189,12 +189,13 @@ end module transform_derived_type_arguments_mod
 
 @pytest.mark.parametrize('all_derived_types', (False, True))
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_expansion_trivial_derived_type_scheduler(frontend, all_derived_types,
-        config, here):
+def test_transform_derived_type_arguments_expansion_trivial_derived_type_scheduler(
+        frontend, all_derived_types, config, here, tmp_path
+):
 
     proj = here / 'sources/projDerivedTypes'
 
-    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend)
+    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend, xmods=[tmp_path])
 
     # Apply transformation
     transformation = DerivedTypeArgumentsTransformation(all_derived_types=all_derived_types)
@@ -226,7 +227,7 @@ def test_transform_derived_type_arguments_expansion_trivial_derived_type_schedul
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_expansion(frontend):
+def test_transform_derived_type_arguments_expansion(frontend, tmp_path):
     fcode = f"""
 module transform_derived_type_arguments_mod
 
@@ -307,7 +308,7 @@ contains
 end module transform_derived_type_arguments_mod
     """.strip()
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     call_tree = [
         ProcedureItem(name='transform_derived_type_arguments_mod#caller', source=source, config={'role': 'driver'}),
@@ -324,10 +325,7 @@ end module transform_derived_type_arguments_mod
         'm', 'n', 't_io%a', 't_io%b', 't_in(i)%a', 't_in(i)%b',
         't_out(i)%a', 't_out(i)%b'
     )
-    if frontend == OMNI:
-        kernel_args = ('m', 'n', 'p_a(1:n)', 'p_b(1:m, 1:n)', 'q_a(:)', 'q_b(:, :)', 'r_a(:)', 'r_b(:, :)')
-    else:
-        kernel_args = ('m', 'n', 'P_a(n)', 'P_b(m, n)', 'Q_a(:)', 'Q_b(:, :)', 'R_a(:)', 'R_b(:, :)')
+    kernel_args = ('m', 'n', 'P_a(n)', 'P_b(m, n)', 'Q_a(:)', 'Q_b(:, :)', 'R_a(:)', 'R_b(:, :)')
 
     call = FindNodes(CallStatement).visit(source['caller'].ir)[0]
     assert call.name == 'kernel'
@@ -343,7 +341,7 @@ end module transform_derived_type_arguments_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_inline_call(frontend):
+def test_transform_derived_type_arguments_inline_call(frontend, tmp_path):
     """
     Verify correct expansion of inline calls to functions
     """
@@ -377,8 +375,10 @@ subroutine driver(arr, n, s, t)
 end subroutine driver
     """.strip()
 
-    source_my_mod = Sourcefile.from_source(fcode_my_mod, frontend=frontend)
-    source_driver = Sourcefile.from_source(fcode_driver, frontend=frontend, definitions=source_my_mod.definitions)
+    source_my_mod = Sourcefile.from_source(fcode_my_mod, frontend=frontend, xmods=[tmp_path])
+    source_driver = Sourcefile.from_source(
+        fcode_driver, frontend=frontend, definitions=source_my_mod.definitions, xmods=[tmp_path]
+    )
 
     kernel = ProcedureItem('my_mod#kernel', config={'role': 'kernel'}, source=source_my_mod)
     driver = ProcedureItem('#driver', config={'role': 'driver'}, source=source_driver)
@@ -400,7 +400,7 @@ end subroutine driver
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_multilevel(frontend):
+def test_transform_derived_type_arguments_multilevel(frontend, tmp_path):
     """
     Verify correct behaviour of the derived type argument flattening when
     used in multi-level call trees. There it is mandatory to traverse the tree from
@@ -445,7 +445,7 @@ contains
 end module transform_derived_type_arguments_multilevel
     """.strip()
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     orig_args = {
         'caller': ('n', 'obj'),
@@ -494,7 +494,7 @@ end module transform_derived_type_arguments_multilevel
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_expansion_nested(frontend):
+def test_transform_derived_type_arguments_expansion_nested(frontend, tmp_path):
     fcode_header = f"""
 module header_mod
     implicit none
@@ -663,11 +663,16 @@ subroutine caller(z)
 end subroutine caller
     """.strip()
 
-    header = Sourcefile.from_source(fcode_header, frontend=frontend)
-    bucket = Sourcefile.from_source(fcode_bucket, frontend=frontend, definitions=header.definitions)
-    layer = Sourcefile.from_source(fcode_layer, frontend=frontend, definitions=header.definitions + bucket.definitions)
-    source = Sourcefile.from_source(fcode_caller, frontend=frontend,
-                                    definitions=header.definitions + bucket.definitions + layer.definitions)
+    header = Sourcefile.from_source(fcode_header, frontend=frontend, xmods=[tmp_path])
+    bucket = Sourcefile.from_source(fcode_bucket, frontend=frontend, definitions=header.definitions, xmods=[tmp_path])
+    layer = Sourcefile.from_source(
+        fcode_layer, frontend=frontend,
+        definitions=header.definitions + bucket.definitions, xmods=[tmp_path]
+    )
+    source = Sourcefile.from_source(
+        fcode_caller, frontend=frontend, xmods=[tmp_path],
+        definitions=header.definitions + bucket.definitions + layer.definitions
+    )
 
     items = {
         'caller': ProcedureItem(
@@ -747,14 +752,9 @@ end subroutine caller
     )
 
     # Check arguments of kernel
-    if frontend == OMNI:
-        assert items['kernel'].ir.arguments == (
-            'm', 'n', 'p_a(1:n)', 'p_b(1:m, 1:n)', 'q_a(:)', 'q_b(:, :)', 'r_a(:)', 'r_b(:, :)', 'c'
-        )
-    else:
-        assert items['kernel'].ir.arguments == (
-            'm', 'n', 'P_a(n)', 'P_b(m, n)', 'Q_a(:)', 'Q_b(:, :)', 'R_a(:)', 'R_b(:, :)', 'c'
-        )
+    assert items['kernel'].ir.arguments == (
+        'm', 'n', 'P_a(n)', 'P_b(m, n)', 'Q_a(:)', 'Q_b(:, :)', 'R_a(:)', 'R_b(:, :)', 'c'
+    )
 
     # Check call arguments in layer
     calls = FindNodes(CallStatement).visit(items['layer'].ir.ir)
@@ -768,16 +768,10 @@ end subroutine caller
     )
 
     # Check arguments of layer
-    if frontend == OMNI:
-        assert items['layer'].ir.arguments == (
-            'm', 'n', 'p_a_a(:)', 'p_a_b(:, :)', 'p_b(1:5)', 'q_a_a(:)', 'q_a_b(:, :)', 'q_b(:)', 'q_constants(:)',
-            'r_a_a(:)', 'r_a_b(:, :)', 'r_b(:)'
-        )
-    else:
-        assert items['layer'].ir.arguments == (
-            'm', 'n', 'p_a_a(:)', 'p_a_b(:, :)', 'p_b(5)', 'q_a_a(:)', 'q_a_b(:, :)', 'q_b(:)', 'q_constants(:)',
-            'r_a_a(:)', 'r_a_b(:, :)', 'r_b(:)'
-        )
+    assert items['layer'].ir.arguments == (
+        'm', 'n', 'p_a_a(:)', 'p_a_b(:, :)', 'p_b(5)', 'q_a_a(:)', 'q_a_b(:, :)', 'q_b(:)', 'q_constants(:)',
+        'r_a_a(:)', 'r_a_b(:, :)', 'r_b(:)'
+    )
 
     # Check imports
     assert 'constants_type' in items['layer'].ir.imported_symbols
@@ -811,7 +805,7 @@ end subroutine caller
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_typebound_proc(frontend):
+def test_transform_derived_type_arguments_typebound_proc(frontend, tmp_path):
     fcode = f"""
 module transform_derived_type_arguments_mod
 
@@ -875,11 +869,13 @@ subroutine driver(some, result)
 end subroutine driver
     """.strip()
 
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     kernel_a = ProcedureItem(name='transform_derived_type_arguments_mod#kernel_a', source=source)
     kernel = ProcedureItem(name='transform_derived_type_arguments_mod#kernel', source=source)
     reduce = ProcedureItem(name='transform_derived_type_arguments_mod#reduce', source=source)
-    source_driver = Sourcefile.from_source(fcode_driver, frontend=frontend, definitions=source.definitions)
+    source_driver = Sourcefile.from_source(
+        fcode_driver, frontend=frontend, definitions=source.definitions, xmods=[tmp_path]
+    )
     driver = ProcedureItem(name='#driver', source=source_driver)
 
     # Check procedure bindings before the transformation
@@ -944,7 +940,7 @@ end subroutine driver
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_import_rename(frontend):
+def test_transform_derived_type_arguments_import_rename(frontend, tmp_path):
     fcode1 = """
 module some_mod
     implicit none
@@ -966,8 +962,8 @@ subroutine some_routine(t)
 end subroutine some_routine
     """.strip()
 
-    source1 = Sourcefile.from_source(fcode1, frontend=frontend)
-    source2 = Sourcefile.from_source(fcode2, frontend=frontend, definitions=source1.definitions)
+    source1 = Sourcefile.from_source(fcode1, frontend=frontend, xmods=[tmp_path])
+    source2 = Sourcefile.from_source(fcode2, frontend=frontend, definitions=source1.definitions, xmods=[tmp_path])
 
     callee = ProcedureItem(name='some_mod#some_routine', source=source1)
     caller = ProcedureItem(name='#some_routine', source=source2)
@@ -992,7 +988,7 @@ end subroutine some_routine
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_optional_named_arg(frontend):
+def test_transform_derived_type_arguments_optional_named_arg(frontend, tmp_path):
     fcode = """
 module some_mod
     implicit none
@@ -1024,7 +1020,7 @@ contains
     end subroutine caller
 end module some_mod
     """.strip()
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     callee = ProcedureItem(name='some_mod#callee', source=source)
     caller = ProcedureItem(name='some_mod#caller', source=source)
@@ -1049,7 +1045,7 @@ end module some_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_recursive(frontend):
+def test_transform_derived_type_arguments_recursive(frontend, tmp_path):
     fcode = """
 module some_mod
     implicit none
@@ -1113,7 +1109,7 @@ contains
     end subroutine caller
 end module some_mod
     """.strip()
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     callee = ProcedureItem(name='some_mod#callee', source=source)
     caller = ProcedureItem(name='some_mod#caller', source=source)
@@ -1169,7 +1165,7 @@ end module some_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_renamed_calls(frontend):
+def test_transform_derived_type_arguments_renamed_calls(frontend, tmp_path):
     fcode_header = """
 module header_mod
     implicit none
@@ -1213,11 +1209,16 @@ subroutine caller(t)
 end subroutine caller
     """.strip()
 
-    source_header = Sourcefile.from_source(fcode_header, frontend=frontend)
-    source_some = Sourcefile.from_source(fcode_some, frontend=frontend, definitions=source_header.definitions)
-    source_other = Sourcefile.from_source(fcode_other, frontend=frontend, definitions=source_header.definitions)
+    source_header = Sourcefile.from_source(fcode_header, frontend=frontend, xmods=[tmp_path])
+    source_some = Sourcefile.from_source(
+        fcode_some, frontend=frontend,
+        definitions=source_header.definitions, xmods=[tmp_path]
+    )
+    source_other = Sourcefile.from_source(
+        fcode_other, frontend=frontend, definitions=source_header.definitions, xmods=[tmp_path]
+    )
     source_caller = Sourcefile.from_source(
-        fcode_caller, frontend=frontend,
+        fcode_caller, frontend=frontend, xmods=[tmp_path],
         definitions=source_header.definitions + source_some.definitions + source_other.definitions
     )
 
@@ -1239,7 +1240,7 @@ end subroutine caller
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_associate_intent(frontend):
+def test_transform_derived_type_arguments_associate_intent(frontend, tmp_path):
     fcode = """
 module some_mod
     implicit none
@@ -1255,7 +1256,7 @@ contains
     end subroutine some_routine
 end module some_mod
     """.strip()
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     variables = FindVariables().visit(source['some_routine'].body)
     assert variables == {'arr(:)', 'arr', 't%arr', 't'}
@@ -1279,7 +1280,7 @@ end module some_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transform_derived_type_arguments_non_array(frontend):
+def test_transform_derived_type_arguments_non_array(frontend, tmp_path):
     fcode = """
 module some_mod
     implicit none
@@ -1303,7 +1304,7 @@ contains
     end subroutine kernel
 end module some_mod
     """.strip()
-    source = Sourcefile.from_source(fcode, frontend=frontend)
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=[tmp_path])
 
     transformation = DerivedTypeArgumentsTransformation()
     source['kernel'].apply(transformation, role='kernel')
@@ -1443,7 +1444,7 @@ end subroutine driver
     (tmp_path/'driver.F90').write_text(fcode4)
 
     scheduler = Scheduler(
-        paths=[tmp_path], config=config, seed_routines=['driver'], frontend=frontend
+        paths=[tmp_path], config=config, seed_routines=['driver'], frontend=frontend, xmods=[tmp_path]
     )
 
     transformation = TypeboundProcedureCallTransformation(duplicate_typebound_kernels=duplicate)
