@@ -5,19 +5,19 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
-from shutil import rmtree
+from pathlib import Path
+# from shutil import rmtree
 import pytest
 import numpy as np
 
-from loki import Subroutine, Module, cgen
+from loki import Subroutine, Module, cgen, cppgen, cudagen, FindNodes, Dimension, Scheduler, read_file
 from loki.build import jit_compile, jit_compile_lib, clean_test, Builder, Obj
 import loki.expression.symbols as sym
 from loki.frontend import available_frontends, OFP
-import loki.ir as ir
+from loki import ir
 
-from loki.transformations.array_indexing import normalize_range_indexing
 from loki.transformations.transpile import FortranCTransformation
-
+from loki.transformations.single_column import SCCLowLevelHoist, SCCLowLevelParametrise
 
 @pytest.fixture(scope='function', name='builder')
 def fixture_builder(tmp_path):
@@ -27,7 +27,8 @@ def fixture_builder(tmp_path):
 
 @pytest.mark.parametrize('case_sensitive', (False, True))
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transpile_case_sensitivity(tmp_path, frontend, case_sensitive):
+@pytest.mark.parametrize('language', ('c', 'cuda'))
+def test_transpile_case_sensitivity(tmp_path, frontend, case_sensitive, language):
     """
     A simple test for testing lowering the case and case-sensitivity
     for specific symbols.
@@ -57,7 +58,7 @@ end subroutine transpile_case_sensitivity
     inline_call_assignment = ir.Assignment(lhs=routine.variable_map['a'], rhs=inline_call)
     routine.body = (routine.body, assignment, call, inline_call_assignment)
 
-    f2c = FortranCTransformation()
+    f2c = FortranCTransformation(language=language, use_c_ptr=language=='cuda')
     f2c.apply(source=routine, path=tmp_path)
     ccode = f2c.c_path.read_text().replace(' ', '').replace('\n', ' ').replace('\r', '').replace('\t', '')
     assert convert_case('transpile_case_sensitivity_c(inta,intsOmE_vAr,intoTher_VaR)', case_sensitive) in ccode
@@ -98,7 +99,6 @@ end subroutine simple_loops
 
     # Generate reference code, compile run and verify
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
     filepath = tmp_path/(f'simple_loops{"_c_ptr" if use_c_ptr else ""}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname='simple_loops')
 
@@ -197,7 +197,6 @@ end subroutine transpile_arguments
 
     # Generate reference code, compile run and verify
     routine = Subroutine.from_source(fcode, frontend=frontend)
-    normalize_range_indexing(routine) # Fix OMNI nonsense
     filepath = tmp_path/(f'transpile_arguments{"_c_ptr" if use_c_ptr else ""}_{frontend}.f90')
     function = jit_compile(routine, filepath=filepath, objname='transpile_arguments')
     a, b, c = function(n, array, array_io, a_io, b_io, c_io)
@@ -250,32 +249,32 @@ def test_transpile_derived_type(tmp_path, builder, frontend, use_c_ptr):
 
     fcode_type = """
 module transpile_type_mod
-  use iso_fortran_env, only: real32, real64
-  implicit none
+    use iso_fortran_env, only: real32, real64
+    implicit none
 
-  type my_struct
-     integer :: a
-     real(kind=real32) :: b
-     real(kind=real64) :: c
-  end type my_struct
+    type my_struct
+        integer :: a
+        real(kind=real32) :: b
+        real(kind=real64) :: c
+    end type my_struct
 end module transpile_type_mod
-"""
+    """.strip()
 
     fcode_routine = """
 subroutine transp_der_type(a_struct)
-  use transpile_type_mod, only: my_struct
-  implicit none
-  type(my_struct), intent(inout) :: a_struct
+    use transpile_type_mod, only: my_struct
+    implicit none
+    type(my_struct), intent(inout) :: a_struct
 
-  a_struct%a = a_struct%a + 4
-  a_struct%b = a_struct%b + 5.
-  a_struct%c = a_struct%c + 6.
+    a_struct%a = a_struct%a + 4
+    a_struct%b = a_struct%b + 5.
+    a_struct%c = a_struct%c + 6.
 end subroutine transp_der_type
-"""
+    """.strip()
     builder.clean()
 
-    module = Module.from_source(fcode_type, frontend=frontend)
-    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend)
+    module = Module.from_source(fcode_type, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend, xmods=[tmp_path])
     refname = f'ref_{routine.name}{"_c_ptr" if use_c_ptr else ""}_{frontend}'
     reference = jit_compile_lib([module, routine], path=tmp_path, name=refname, builder=builder)
 
@@ -322,34 +321,34 @@ def test_transpile_associates(tmp_path, builder, frontend, use_c_ptr):
 
     fcode_type = """
 module assoc_type_mod
-  use iso_fortran_env, only: real32, real64
-  implicit none
+    use iso_fortran_env, only: real32, real64
+    implicit none
 
-  type my_struct
-     integer :: a
-     real(kind=real32) :: b
-     real(kind=real64) :: c
-  end type my_struct
+    type my_struct
+        integer :: a
+        real(kind=real32) :: b
+        real(kind=real64) :: c
+    end type my_struct
 end module assoc_type_mod
-"""
+    """.strip()
 
     fcode_routine = """
 subroutine transp_assoc(a_struct)
-  use assoc_type_mod, only: my_struct
-  implicit none
-  type(my_struct), intent(inout) :: a_struct
+    use assoc_type_mod, only: my_struct
+    implicit none
+    type(my_struct), intent(inout) :: a_struct
 
-  associate(a_struct_a=>a_struct%a, a_struct_b=>a_struct%b,&
-   & a_struct_c=>a_struct%c)
-  a_struct%a = a_struct_a + 4.
-  a_struct_b = a_struct%b + 5.
-  a_struct_c = a_struct_a + a_struct%b + a_struct_c
-  end associate
+    associate(a_struct_a=>a_struct%a, a_struct_b=>a_struct%b,&
+            & a_struct_c=>a_struct%c)
+        a_struct%a = a_struct_a + 4.
+        a_struct_b = a_struct%b + 5.
+        a_struct_c = a_struct_a + a_struct%b + a_struct_c
+    end associate
 end subroutine transp_assoc
-"""
+    """.strip()
 
-    module = Module.from_source(fcode_type, frontend=frontend)
-    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend)
+    module = Module.from_source(fcode_type, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend, xmods=[tmp_path])
     refname = f'ref_{routine.name}{"_c_ptr" if use_c_ptr else ""}_{frontend}'
     reference = jit_compile_lib([module, routine], path=tmp_path, name=refname, builder=builder)
 
@@ -427,34 +426,34 @@ def test_transpile_module_variables(tmp_path, builder, frontend, use_c_ptr):
 
     fcode_type = """
 module mod_var_type_mod
-  use iso_fortran_env, only: real32, real64
-  implicit none
+    use iso_fortran_env, only: real32, real64
+    implicit none
 
-  save
+    save
 
-  integer :: PARAM1
-  real(kind=real32) :: param2
-  real(kind=real64) :: param3
+    integer :: PARAM1
+    real(kind=real32) :: param2
+    real(kind=real64) :: param3
 end module mod_var_type_mod
-"""
+    """.strip()
 
     fcode_routine = """
 subroutine transp_mod_var(a, b, c)
-  use iso_fortran_env, only: real32, real64
-  use mod_var_type_mod, only: PARAM1, param2, param3
+    use iso_fortran_env, only: real32, real64
+    use mod_var_type_mod, only: PARAM1, param2, param3
 
-  integer, intent(out) :: a
-  real(kind=real32), intent(out) :: b
-  real(kind=real64), intent(out) :: c
+    integer, intent(out) :: a
+    real(kind=real32), intent(out) :: b
+    real(kind=real64), intent(out) :: c
 
-  a = 1 + PARAM1  ! Ensure downcasing is done right
-  b = 1. + param2
-  c = 1. + param3
+    a = 1 + PARAM1  ! Ensure downcasing is done right
+    b = 1. + param2
+    c = 1. + param3
 end subroutine transp_mod_var
-"""
+    """.strip()
 
-    module = Module.from_source(fcode_type, frontend=frontend)
-    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend)
+    module = Module.from_source(fcode_type, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode_routine, definitions=module, frontend=frontend, xmods=[tmp_path])
     refname = f'ref_{routine.name}{"_c_ptr" if use_c_ptr else ""}_{frontend}'
     reference = jit_compile_lib([module, routine], path=tmp_path, name=refname, builder=builder)
 
@@ -818,8 +817,8 @@ subroutine inline_elemental(v1, v2, v3)
 end subroutine inline_elemental
 """
     # Generate reference code, compile run and verify
-    module = Module.from_source(fcode_module, frontend=frontend)
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    module = Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     refname = f'ref_{routine.name}{"_c_ptr" if use_c_ptr else ""}_{frontend}'
     reference = jit_compile_lib([module, routine], path=tmp_path, name=refname, builder=builder)
 
@@ -831,7 +830,7 @@ end subroutine inline_elemental
     (tmp_path/f'{routine.name}.f90').unlink()
 
     # Now transpile with supplied elementals but without module
-    routine = Subroutine.from_source(fcode, definitions=module, frontend=frontend)
+    routine = Subroutine.from_source(fcode, definitions=module, frontend=frontend, xmods=[tmp_path])
 
     f2c = FortranCTransformation(inline_elementals=True, use_c_ptr=use_c_ptr)
     f2c.apply(source=routine, path=tmp_path)
@@ -893,8 +892,8 @@ subroutine inline_elementals_rec(v1, v2, v3)
 end subroutine inline_elementals_rec
 """
     # Generate reference code, compile run and verify
-    module = Module.from_source(fcode_module, frontend=frontend)
-    routine = Subroutine.from_source(fcode, frontend=frontend)
+    module = Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, xmods=[tmp_path])
     refname = f'ref_{routine.name}{"_c_ptr" if use_c_ptr else ""}_{frontend}'
     reference = jit_compile_lib([module, routine], path=tmp_path, name=refname, builder=builder)
 
@@ -906,7 +905,7 @@ end subroutine inline_elementals_rec
     (tmp_path/f'{routine.name}.f90').unlink()
 
     # Now transpile with supplied elementals but without module
-    routine = Subroutine.from_source(fcode, definitions=module, frontend=frontend)
+    routine = Subroutine.from_source(fcode, definitions=module, frontend=frontend, xmods=[tmp_path])
 
     f2c = FortranCTransformation(inline_elementals=True, use_c_ptr=use_c_ptr)
     f2c.apply(source=routine, path=tmp_path)
@@ -980,17 +979,20 @@ end subroutine transpile_expressions
 
 @pytest.mark.parametrize('use_c_ptr', (False, True))
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transpile_call(tmp_path, frontend, use_c_ptr):
+@pytest.mark.parametrize('language', ('c', 'cuda'))
+@pytest.mark.parametrize('chevron', (False, True))
+def test_transpile_call(tmp_path, frontend, use_c_ptr, language, chevron):
     fcode_module = """
 module transpile_call_kernel_mod
   implicit none
 contains
 
-  subroutine transpile_call_kernel(a, b, c, arr1, len)
+  subroutine transpile_call_kernel(a, b, c, arr1, arr2, len)
     integer, intent(inout) :: a, c
     integer, intent(in) :: b
     integer, intent(in) :: len
     integer, intent(inout) :: arr1(len, len)
+    integer, intent(in) :: arr2(len, len)
     a = b
     c = b
   end subroutine transpile_call_kernel
@@ -1006,12 +1008,15 @@ subroutine transpile_call_driver(a)
     integer :: arr2(len, len)
     integer :: b
     b = 2 * len
-    call transpile_call_kernel(a, b, arr2(1, 1), arr1, len)
+    call transpile_call_kernel(a, b, arr2(1, 1), arr1, arr2, len)
 end subroutine transpile_call_driver
 """
-    module = Module.from_source(fcode_module, frontend=frontend)
-    routine = Subroutine.from_source(fcode, frontend=frontend, definitions=module)
-    f2c = FortranCTransformation(use_c_ptr=use_c_ptr, path=tmp_path)
+    module = Module.from_source(fcode_module, frontend=frontend, xmods=[tmp_path])
+    routine = Subroutine.from_source(fcode, frontend=frontend, definitions=module, xmods=[tmp_path])
+    if chevron:
+        calls = FindNodes(ir.CallStatement).visit(routine.body)
+        calls[0]._update(chevron=(sym.IntLiteral(1), sym.IntLiteral(1)))
+    f2c = FortranCTransformation(use_c_ptr=use_c_ptr, path=tmp_path, language=language)
     f2c.apply(source=module.subroutine_map['transpile_call_kernel'], path=tmp_path, role='kernel')
     ccode_kernel = f2c.c_path.read_text().replace(' ', '').replace('\n', '')
     f2c.apply(source=routine, path=tmp_path, role='kernel')
@@ -1021,13 +1026,73 @@ end subroutine transpile_call_driver
     # check for applied Dereference
     assert "(*a)=b;" in ccode_kernel
     assert "(*c)=b;" in ccode_kernel
-    # check for applied Reference
-    assert "transpile_call_kernel((&a),b,(&arr2[" in ccode_driver
+    # check for applied 'const' and 'restrict'/'__restrict__'
+    if language == 'cuda':
+        assert 'int*a,intb,int*c,int*__restrict__arr1,constint*__restrict__arr2' in ccode_kernel
+    else:
+        assert 'int*a,intb,int*c,int*restrictarr1,int*restrictarr2' in ccode_kernel
+    # check for applied Reference and chevron
+    if chevron and language == 'cuda':
+        assert "transpile_call_kernel<<<1,1>>>((&a),b,(&arr2[" in ccode_driver
+    else:
+        assert "transpile_call_kernel((&a),b,(&arr2[" in ccode_driver
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
+@pytest.mark.parametrize('header', (False, True))
+@pytest.mark.parametrize('guards', (False, True))
+@pytest.mark.parametrize('extern', (False, True))
+@pytest.mark.parametrize('guard_name', (None, 'random_guard_name'))
+def test_transpile_simple_routine(tmp_path, frontend, codegen, guards, guard_name,
+        header, extern):
+    """
+    Test correct transpilation of functions in C transpilation with a focus
+    on code-gen options.
+    """
+
+    fcode = """
+subroutine add(a, b, result)
+    real, intent(in) :: a, b
+    real, intent(out) :: result
+
+    result = a + b
+end subroutine add
+""".strip()
+
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    f2c = FortranCTransformation()
+    f2c.apply(source=routine, path=tmp_path)
+
+    c_routine = codegen(routine, guards=guards, guard_name=guard_name,
+            header=header, extern=extern)
+    if extern and codegen in (cppgen, cudagen):
+        assert 'extern "C"' in c_routine
+    else:
+        assert 'extern "C"' not in c_routine
+    if header:
+        assert 'void add(double a, double b, double result);' in c_routine
+    else:
+        assert 'void add(double a, double b, double result) {' in c_routine
+    if guards:
+        if guard_name is None:
+            assert '#ifndef ADD_H' in c_routine
+            assert '#define ADD_H' in c_routine
+            assert '#endif' in c_routine
+        else:
+            assert '#ifndef random_guard_name' in c_routine
+            assert '#define random_guard_name' in c_routine
+            assert '#endif' in c_routine
+    else:
+        assert '#ifndef' not in c_routine
+        assert '#define' not in c_routine
+        assert '#endif' not in c_routine
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('f_type', ['integer', 'real'])
-def test_transpile_inline_functions(tmp_path, frontend, f_type):
+@pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
+def test_transpile_inline_functions(tmp_path, frontend, f_type, codegen):
     """
     Test correct transpilation of functions in C transpilation.
     """
@@ -1046,14 +1111,15 @@ end function add
     f2c.apply(source=routine, path=tmp_path)
 
     f_type_map = {'integer': 'int', 'real': 'double'}
-    c_routine = cgen(routine)
+    c_routine = codegen(routine)
     assert 'return add;' in c_routine
     assert f'{f_type_map[f_type]} add(' in c_routine
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('f_type', ['integer', 'real'])
-def test_transpile_inline_functions_return(tmp_path, frontend, f_type):
+@pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
+def test_transpile_inline_functions_return(tmp_path, frontend, f_type, codegen):
     """
     Test correct transpilation of functions in C transpilation.
     """
@@ -1072,13 +1138,14 @@ end function add
     f2c.apply(source=routine, path=tmp_path)
 
     f_type_map = {'integer': 'int', 'real': 'double'}
-    c_routine = cgen(routine)
+    c_routine = codegen(routine)
     assert 'return res;' in c_routine
     assert f'{f_type_map[f_type]} add(' in c_routine
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transpile_multiconditional(tmp_path, builder, frontend):
+@pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
+def test_transpile_multiconditional(tmp_path, builder, frontend, codegen):
     """
     A simple test to verify multiconditionals/select case statements.
     """
@@ -1122,7 +1189,7 @@ end subroutine multi_cond
     f2c.apply(source=routine, path=tmp_path)
 
     # check whether 'switch' statement is within C code
-    assert 'switch' in cgen(routine)
+    assert 'switch' in codegen(routine)
 
     # compile C version
     libname = f'fc_{routine.name}_{frontend}'
@@ -1181,3 +1248,184 @@ end subroutine transpile_multi_conditional_range
     f2c = FortranCTransformation()
     with pytest.raises(NotImplementedError):
         f2c.apply(source=routine, path=tmp_path)
+
+@pytest.fixture(scope='module', name='horizontal')
+def fixture_horizontal():
+    return Dimension(name='horizontal', size='nlon', index='jl', bounds=('start', 'iend'))
+
+
+@pytest.fixture(scope='module', name='vertical')
+def fixture_vertical():
+    return Dimension(name='vertical', size='nz', index='jk')
+
+
+@pytest.fixture(scope='module', name='blocking')
+def fixture_blocking():
+    return Dimension(name='blocking', size='nb', index='b')
+
+
+@pytest.fixture(scope='module', name='here')
+def fixture_here():
+    return Path(__file__).parent
+
+
+@pytest.fixture(name='config')
+def fixture_config():
+    """
+    Default configuration dict with basic options.
+    """
+    return {
+        'default': {
+            'mode': 'idem',
+            'role': 'kernel',
+            'expand': True,
+            'strict': False,  # cudafor import
+        },
+        'routines': {
+            'driver': {'role': 'driver'}
+        }
+    }
+
+def remove_whitespace_linebreaks(text):
+    return text.replace(' ', '').replace('\n', ' ').replace('\r', '').replace('\t', '').lower()
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_scc_cuda_parametrise(tmp_path, here, frontend, config, horizontal, vertical, blocking):
+    """
+    Test SCC-CUF transformation type 0, thus including parametrising (array dimension(s))
+    """
+
+    proj = here / '../../tests/sources/projSccCuf/module'
+
+    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend)
+
+    dic2p = {'nz': 137}
+    cuda_transform = SCCLowLevelParametrise(
+        horizontal=horizontal, vertical=vertical, block_dim=blocking,
+        transformation_type='parametrise',
+        dim_vars=(vertical.size,), as_kwarguments=True, remove_vector_section=True,
+        use_c_ptr=True, dic2p=dic2p, path=here, mode='cuda'
+    )
+    scheduler.process(transformation=cuda_transform)
+    f2c_transformation = FortranCTransformation(path=tmp_path, language='cuda', use_c_ptr=True)
+    scheduler.process(transformation=f2c_transformation)
+
+    fc_kernel = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_fc.F90'))
+    c_kernel = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c.c'))
+    c_kernel_header = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c.h'))
+    c_kernel_launch = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c_launch.h'))
+    c_device = remove_whitespace_linebreaks(read_file(tmp_path/'device_c.c'))
+    c_elemental_device = remove_whitespace_linebreaks(read_file(tmp_path/'elemental_device_c.c'))
+
+    calls = FindNodes(ir.CallStatement).visit(scheduler["driver_mod#driver"].ir.body)
+    assert len(calls) == 3
+    for call in calls:
+        assert str(call.name).lower() == 'kernel'
+        assert call.pragma[0].keyword == 'loki'
+        assert 'removed_loop' in call.pragma[0].content
+    # kernel_fc.F90
+    assert '!$acchost_datause_device(q,t,z)' in fc_kernel
+    assert 'kernel_iso_c(start,nlon,c_loc(q),c_loc(t),c_loc(z),nb,tot,iend)' in fc_kernel
+    assert 'bind(c,name="kernel_c_launch")' in fc_kernel
+    assert 'useiso_c_binding' in fc_kernel
+    # kernel_c.c
+    assert '#include<cuda.h>' in c_kernel
+    assert '#include<cuda_runtime.h>' in c_kernel
+    assert '#include"kernel_c.h"' in c_kernel
+    assert '#include"kernel_c_launch.h"' in c_kernel
+    assert 'include"elemental_device_c.h"' in c_kernel
+    assert 'include"device_c.h"' in c_kernel
+    assert '__global__voidkernel_c' in c_kernel
+    assert 'jl=threadidx.x;' in c_kernel
+    assert 'b=blockidx.x;' in c_kernel
+    # kernel_c.h
+    assert '__global__voidkernel_c' in c_kernel_header
+    assert 'jl=threadidx.x;' not in c_kernel_header
+    assert 'b=blockidx.x;' not in c_kernel_header
+    # kernel_c_launch.h
+    assert 'extern"c"' in c_kernel_launch
+    assert 'voidkernel_c_launch(' in c_kernel_launch
+    assert 'structdim3blockdim;' in c_kernel_launch
+    assert 'structdim3griddim;' in c_kernel_launch
+    assert 'griddim=dim3(' in c_kernel_launch
+    assert 'blockdim=dim3(' in c_kernel_launch
+    assert 'kernel_c<<<griddim,blockdim>>>(' in c_kernel_launch
+    assert 'cudadevicesynchronize();' in c_kernel_launch
+    # device_c.c
+    assert '#include<cuda.h>' in c_device
+    assert '#include<cuda_runtime.h>' in c_device
+    assert '#include"device_c.h"' in c_device
+    assert '#include<cuda.h>' in c_elemental_device
+    assert '#include<cuda_runtime.h>' in c_elemental_device
+    assert '#include"elemental_device_c.h"' in c_elemental_device
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_scc_cuda_hoist(tmp_path, here, frontend, config, horizontal, vertical, blocking):
+    """
+    Test SCC-CUF transformation type 0, thus including parametrising (array dimension(s))
+    """
+
+    proj = here / '../../tests/sources/projSccCuf/module'
+
+    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend)
+
+    cuda_transform = SCCLowLevelHoist(
+        horizontal=horizontal, vertical=vertical, block_dim=blocking,
+        transformation_type='parametrise',
+        dim_vars=(vertical.size,), as_kwarguments=True, remove_vector_section=True,
+        use_c_ptr=True, path=here, mode='cuda'
+    )
+    scheduler.process(transformation=cuda_transform)
+    f2c_transformation = FortranCTransformation(path=tmp_path, language='cuda', use_c_ptr=True)
+    scheduler.process(transformation=f2c_transformation)
+
+    fc_kernel = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_fc.F90'))
+    c_kernel = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c.c'))
+    c_kernel_header = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c.h'))
+    c_kernel_launch = remove_whitespace_linebreaks(read_file(tmp_path/'kernel_c_launch.h'))
+    c_device = remove_whitespace_linebreaks(read_file(tmp_path/'device_c.c'))
+    c_elemental_device = remove_whitespace_linebreaks(read_file(tmp_path/'elemental_device_c.c'))
+
+    calls = FindNodes(ir.CallStatement).visit(scheduler["driver_mod#driver"].ir.body)
+    assert len(calls) == 3
+    for call in calls:
+        assert str(call.name).lower() == 'kernel'
+        assert call.pragma[0].keyword == 'loki'
+        assert 'removed_loop' in call.pragma[0].content
+    # kernel_fc.F90
+    assert '!$acchost_datause_device(q,t,z,local_z,device_local_x)' in fc_kernel
+    assert 'kernel_iso_c(start,nlon,nz,c_loc(q),c_loc(t),c_loc(z)' in fc_kernel
+    assert 'c_loc(z),nb,tot,iend,c_loc(local_z),c_loc(device_local_x))' in fc_kernel
+    assert 'bind(c,name="kernel_c_launch")' in fc_kernel
+    assert 'useiso_c_binding' in fc_kernel
+    # kernel_c.c
+    assert '#include<cuda.h>' in c_kernel
+    assert '#include<cuda_runtime.h>' in c_kernel
+    assert '#include"kernel_c.h"' in c_kernel
+    assert '#include"kernel_c_launch.h"' in c_kernel
+    assert '#include"elemental_device_c.h"' in c_kernel
+    assert '#include"device_c.h"' in c_kernel
+    assert '__global__voidkernel_c' in c_kernel
+    assert 'jl=threadidx.x;' in c_kernel
+    assert 'b=blockidx.x;' in c_kernel
+    # kernel_c.h
+    assert '__global__voidkernel_c' in c_kernel_header
+    assert 'jl=threadidx.x;' not in c_kernel_header
+    assert 'b=blockidx.x;' not in c_kernel_header
+    # kernel_c_launch.h
+    assert 'extern"c"' in c_kernel_launch
+    assert 'voidkernel_c_launch(' in c_kernel_launch
+    assert 'structdim3blockdim;' in c_kernel_launch
+    assert 'structdim3griddim;' in c_kernel_launch
+    assert 'griddim=dim3(' in c_kernel_launch
+    assert 'blockdim=dim3(' in c_kernel_launch
+    assert 'kernel_c<<<griddim,blockdim>>>(' in c_kernel_launch
+    assert 'cudadevicesynchronize();' in c_kernel_launch
+    # device_c.c
+    assert '#include<cuda.h>' in c_device
+    assert '#include<cuda_runtime.h>' in c_device
+    assert '#include"device_c.h"' in c_device
+    # elemental_device_c.c
+    assert '#include<cuda.h>' in c_elemental_device
+    assert '#include<cuda_runtime.h>' in c_elemental_device
+    assert '#include"elemental_device_c.h"' in c_elemental_device
